;******************************************************************
;graphical display driver & font-table (5*8dots)
;******************************************************************
;
;hard-wire:
;pb0...pb7 == db0...db7
;
;pd1 == w/r
;pd2 == cs1
;pd3 == cs2
;pd4 == (pwm helligkeit)
;pd5 == (pwm kontrast)
;pd6 == d/i
;pd7 == e
;
;
;a) text-mode: fonts lt. table
;
;b) grafik-mode: absolute & relative Koordinaten
	
;****************************************************************
;definitionen
;****************************************************************
;
;variablen:
.equ	xPOSabs	=	0x90		;absolute x-position
.equ	yPOSabs	=	0x91		;absolute y-position
.equ	xPAGEpos	=	0x92		;x-page position
.equ	xCHARsiz	=	0x93		;char x-size
.equ	yCHARsiz	=	0x94		;char y-size
;
.equ	stZpointL	=	0x95		;z-pointer l
.equ	stZpointH	=	0x96		;z-pointer h
.equ	stYpointL	=	0x97		;y-pointer l
.equ	stYpointH	=	0x98		;y-pointer h
;	
.equ	CNTx		=	0x99		;x-pixel counter
.equ	CNTy		=	0x9a		;y-pixel counter	

;neue variablen fr flexible
;fontdarstellung & positionierung

;a) page-berechnung (basis: txtBOXpos0y + yCHARsize) 
.equ	firstPAGEmask	=	0x9b	;erste page mask innerhalb des bereichs
.equ  lastPAGEmask	=	0x9c	;letzte page mask innerhalb des bereichs
.equ	cntPAGE		=	0x9d	;page counter
.equ	firstPAGE		=	0x9e	;(#)first page
.equ	lastPAGE		=	0x9f	;(#)last page
.equ	effPAGErange	=	0xa0	;(#)effective page-range 1-8
.equ	SFTfactor		=	0xa1	;(#)shift factor
.equ	CHARfactor		=	0xa2	;(#)char vergrerungsfaktor
;
.equ	SFTregST		=	0xaa	;startadresse 56-bit schieberegister

;
;textbox ab adresse: 0x...
.equ	txtBOXstart	=	0x100	;startadress
;txtBOXset0FL(adr+(5*0)+0)	=	text-box0 status flags
;txtBOXpos0x (adr+(5*0)+1)	=	absolute x-position text-box0
;txtBOXpos0y (adr+(5*0)+2)	=	absolute y-position text-box0
;txtBOXsiz0x (adr+(5*0)+3)	=	relative x-lnge text-box0
;txtBOXsiz0y (adr+(5*0)+4)	=	relative y-hhe text-box0
;
;txtBOXset1FL (adr+(5*1)+0)	=	text-box1 status flags
;txtBOXpos1x (adr+(5*1)+1)	=	absolute x-position text-box1
;txtBOXpos1y (adr+(5*1)+2)	=	absolute y-position text-box1
;txtBOXsiz1x (adr+(5*1)+3)	=	relative x-lnge text-box1
;txtBOXsiz1y (adr+(5*1)+4)	=	relative y-hhe text-box1
;
;usw....

;****************************************************************
;
;flags <txtBOXset{x}FL>:
;
.equ	txtBOXact	=	0	;text-box aktiv
.equ	txtBOXclr	=	1	;text-box clear
.equ	txtBOXsftR	=	2	;text-box shift right (y+1)
.equ	txtBOXsftL	=	3	;text-box shift left  (y-1)
.equ	txtBOXsftU	=	4	;text-box shift up    (x-1)
.equ	txtBOXsftD	=	5	;text-box shift down  (x+1)
.equ	txtBOXcont	=	6	;text-box contour (rahmen)
.equ	txtBOXinv	=	7	;text-box invertieren

;***************************************************************
;
;mask:
;ACHTUNG: nach adressierung ein dummy-read erforderlich !
;a) write instructions
;- D/I = 0 
;- W/R = 0
;
.equ	IdisON	=	0x3f	;instruktion display ON
.equ	IdisOFF	=	0x3e	;instruktion display OFF
.equ	IpageADR	=	0xb8	;instruktion set x-page adress 0 (add 1-7 for following pages!)
.equ	IyADR		=	0x40	;instruktion set y-line adress 0 (add 1-63 for following lines!)
.equ	IstLINE	=	0xc0	;instruktion set display start-line 0 (add 1-63 for following lines!
.equ	IleftYpage	=	0x8	;d/i,w/r,cs1,cs2 mask for left y-page	
.equ	IrightYpage =	0x4	; "          "        "   right y-page
;
;b) write data
;- D/I =1
;- W/R =0
;
;c) read instructions
;- D/I =0 
;- W/R =1
;
;d) read data
;- D/I =1
;- W/R =1
;
;*************************************************************
;
;display-flags:
.equ	Dbusy	=	7	;pb7	;busy-flag
.equ	Don	=	5	;pb5	;on-flag
.equ	Dres	=	4	;pb4	;reset-flag
;
.equ	Dena	=	7	;pd7	;e
.equ	Did	=	6	;pd6	;d/i
.equ	Dwr	=	1	;pd1  ;w/r
.equ	cs1	=	2	;pd2	;linke displayhlfte
.equ	cs2	=	3	;pd3	;rechte displayhlfte



;*************************************************************	
;
;z.Zt keine Interrupts
.org	0x10
	;system init
_initSYS:
	;stack-range
	ldi	r16,high(ramend)
	out	sph,r16
	ldi	r16,low(ramend)
	out	spl,r16
	;port-setting
	ldi	r16,0
	out	ddrb,r16	;data 0-7 (default=input)
	out	portb,r16
	;
	ldi	r16,0xfe	;<pd7=E><pd6=d|I><pd5=pwmHELL><pd4=pwmKONT><pd3=cs2><pd2=cs1><pd1=w|r><pd0=rx>
	out	ddrd,r16
	ldi	r16,0x82	;E=1,D|I=0,W|R=1,cs1=0,cs2=0
	out	portd,r16
	;display init
	_initDIS:
		call	RDbusy	;busy-flag lesen
		ldi	r20,IdisON	;beide display hlften on
		call	WRinstr	;write instruction 
		call	RDbusy	;busy-flag lesen
	
		;set display-ram startadressen
		ldi	r20,IpageADR	
		call	WRinstr	;page adresse =0
		call	RDbusy
		;						
		ldi	r20,IyADR
		call	WRinstr	;y-line-adresse =0
		call	RDbusy
		;				
		ldi	r20,IstLINE
		call	WRinstr	;x-line-adresse =0
		call	RDbusy
		;
		;starte mit linker displayhlfte...
		ldi	r20,IleftYpage
		call	WRnewYpage		;y-adresse =0
		call	RDbusy


	

	;**************************************************************************

	;test
	;write fonts to display
_WRITEfonts:
	;set text startadresse
	ldi	yh,high(_txtTABLE*2)
	ldi	yl,low(_txtTABLE*2)
	sts	stYpointL,yl
	sts	stYpointH,yh
	;
	clr	r20
	sts	yPOSabs,r20	;absolute y-position =0
	sts	xPOSabs,r20	;absolute x-position =0
	sts	xPAGEpos,r20;x-page position =0
	;standard font gre
	ldi	r20,6
	sts	xCHARsiz,r20;char x-size 6 pix
	ldi	r20,8
	sts	yCHARsiz,r20;char y-size 8 pix
	;
				_LDtxtCHAR:
					;adressiere character
					lds	zh,stYpointH	;textbuchstaben
					lds	zl,stYpointL	;laden...
					lpm	r16,z+	
					sts	stYpointL,zl
					sts	stYpointH,zh
					tst	r16			;0= zeilenumbruch !
					brne	_LDfontBMP
					jmp	NEWxpage		;...neue displayzeile !



_LDfontBMP:
	;set font startadresse
	ldi	zh,high(font0x20_lines*2)
	ldi	zl,low(font0x20_lines*2)
	subi	r16,0x20		;es geht bei 0x20 los !
	ldi	r17,6			;bmp x-size
	mul	r16,r17
	add	zl,r0
	adc	zh,r1
	ldi	r17,7			;bmp x-size+1
	sts	CNTx,r17
			_LDfontLOOP:
				lds	r17,CNTx
				dec	r17
				tst	r17
				brne	_LDfontLOOPgo
				jmp	_LDtxtCHAR
	;
_LDfontLOOPgo:
	sts	CNTx,r17
	lpm	r20,z+
	;
	call	WRdata	;schreiben daten ins display
	call	RDbusy	;busy-flag testen...

	;ypos +1
	;info: y-pos  >63 dann rechte displayseite cs1=1 / cs2=0
	;      y-pos  >127 dann linke displayseite cs1=0 / cs2=1 & x-page +1
	;      x-page >7 = fertig !
	lds	r16,yPOSabs
	inc	r16
	sts	yPOSabs,r16
	cpi	r16,64
	breq	RIGHTside
	cpi	r16,128
	breq	goLEFTside
	rjmp	LDloopOK

			

							;b)right side
							RIGHTside:
													
								;y-pos =0 & rechte y-page
								ldi	r20,IrightYpage
								call	WRnewYpage		;y-adresse =0
								call	RDbusy	

								rjmp	LDloopOK



									;c)go left side (y-line=0)& x-page +1
								goLEFTside:																				
									;x-page position+1
									lds	r16,xPAGEpos
									inc	r16
									cpi	r16,8
									brsh	GOready
									sts	xPAGEpos,r16
									;
									ldi	r20,IpageADR	;adressierungs instruction (0) laden
									add	r20,r16
									;fr beide y-pages
									cbi	portd,cs1
									cbi	portd,cs2
									call	WRinstr		;neue x-adresse
									call	RDbusy	
									;
									ldi	r20,IleftYpage
									call	WRnewYpage		;y-adresse =0
									call	RDbusy

									;y-pos =0
									clr	r20
									sts	yPOSabs,r20											
									rjmp	_LDtxtCHAR
									;
									;
;
LDloopOK:
jmp	_LDfontLOOP


					;d) new x-page+1 & go left side (y-line=0) & new character
				NEWxpage:
					;
					;x-page position+1
					lds	r16,xPAGEpos
					inc	r16
					cpi	r16,8
					brsh	GOready
					sts	xPAGEpos,r16
					;
					ldi	r20,IpageADR	;adressierungs instruction (0) laden
					add	r20,r16
					;fr beide y-pages
					cbi	portd,cs1
					cbi	portd,cs2

					call	WRinstr		;neue x-adresse
					call	RDbusy
					
					ldi	r20,IleftYpage
					call	WRnewYpage		;y-adresse =0
					call	RDbusy
			

						
					;y-pos =0
					clr	r20
					sts	yPOSabs,r20
					
					jmp	_LDtxtCHAR
					;


	



	;ENDE

GOready:
	rjmp	GOready









;****************************************************************
;sub
;****************************************************************

;read busy-flag
RDbusy:
	ldi	r16,0
	out	ddrb,r16	;portb input
	cbi	portd,Dena	;enable low
	call	wt5cycle	;einige cycles warten...
	in	r16,portd
	andi	r16,0xbd	;nur d/i und w/r ausblenden
	ori	r16,0x2	;d/i=0  w/r=1
	out	portd,r16
RDbusyGO:
	call	wt5cycle	;einige cycles warten...
	sbi	portd,Dena	;enable high
	call	wt5cycle	;einige cycles warten...
	cbi	portd,Dena	;enable low
	in	r16,pinb	;port-data lesen
	sbrc	r16,Dbusy	;0=not busy
	rjmp	RDbusyGO	;1=busy
	ret

;***************************************************************
;
;write instruction
WRinstr:
	ldi	r16,0xff
	out	ddrb,r16	;portb output
	call	wt5cycle	;einige cycles warten...
	in	r16,portd
	andi	r16,0xbd	;nur d/i und w/r ausblenden
	ori	r16,0		;d/i=0  w/r=0
	out	portd,r16
	call	wt5cycle	;einige cycles warten...
	sbi	portd,Dena	;enable high
	out	portb,r20	;port-data schreiben
	call	wt5cycle	;einige cycles warten...
	cbi	portd,Dena	;enable low
	ret

;***************************************************************

;write instruction new y-page & y-adesss=0
WRnewYpage:
	ldi	r16,0xff
	out	ddrb,r16	;portb output
	call	wt5cycle	;einige cycles warten...
	in	r16,portd
	andi	r16,0xb1	; d/i und w/r  cs1 & cs2 ausblenden
	or	r16,r20	;d/i=0  w/r=0 
	out	portd,r16
	call	wt5cycle	;einige cycles warten...
	sbi	portd,Dena	;enable high
	ldi	r20,IyADR	;y-adresse=0
	out	portb,r20	;port-data schreiben
	call	wt5cycle	;einige cycles warten...
	cbi	portd,Dena	;enable low
	ret


;*************************************************************
;
;write data
WRdata:
	ldi	r16,0xff
	out	ddrb,r16	;portb output
	call	wt5cycle	;einige cycles warten...
	in	r16,portd
	andi	r16,0xbd	;nur d/i und w/r ausblenden
	ori	r16,0x40		;d/i=1  w/r=0
	out	portd,r16
	call	wt5cycle	;einige cycles warten...
	sbi	portd,Dena	;enable high
	out	portb,r20	;port-data schreiben
	call	wt5cycle	;einige cycles warten...
	cbi	portd,Dena	;enable low
	ret



;****************************************************************
;wait-loop
wt5cycle:
	ldi	r16,15	;4
	wt5cycleLOOP:
		tst	r16
		breq	wt5cycleBK
		dec	r16
		rjmp	wt5cycleLOOP
		wt5cycleBK:
			ret


;*****************************************************************





.include "m16def.inc"
.include "font_table.asm"
.include "txtTABLE.asm"



.exit